#include "mainwindow.h"
#include <QErrorMessage>
#include <QPushButton>
#include <QFont>
#include <QtXmlPatterns>
#include <QContextMenuEvent>
#include <addressbookparser.h>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
    , currnet_file(nullptr)
    , currnet_pos(nullptr)
{
    ui->setupUi(this);
    treeWidget = new QTreeWidget(this);
    treeWidget->setHeaderLabels(QStringList() << "Наименование" << "Значение");
    treeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
    connect(treeWidget, SIGNAL(customContextMenuRequested(const QPoint&)),
            this, SLOT(slotCustomMenuRequested(const QPoint&)));
    menuCustom = new QMenu(this);
    menuCustom->addAction("&Сделать активным", this, SLOT(ActActive()));
    menuCustom->addAction("&Закрыть", this, SLOT(CloseFile()));
    textEdit = new QTextEdit(this);
    QMenu *menu = new QMenu("&Menu", this);
    QPushButton *putton_download = new QPushButton("Загрузить .xq",this);
    putton_download->setFont(*(new QFont("Arial", 12)));
    connect(putton_download, SIGNAL(clicked()), SLOT(OpenQuery()));
    QPushButton *putton = new QPushButton("Выполнить XQuery запрос",this);
    putton->setFont(*(new QFont("Arial", 12)));
    connect(putton, SIGNAL(clicked()), SLOT(ExecuteQuery()));
    menu->addAction("&Open File", this, SLOT(OpenFile()), Qt::CTRL + Qt::Key_O);
    menu->addAction("&Close All", this, SLOT(ClearTree()));
    menu->addAction("&Execute XQuery", this, SLOT(OpenQuery()), Qt::CTRL + Qt::Key_E);
    menu->addSeparator();
    menu->addAction("&Exit", this, SLOT(CloseWindow()), Qt::CTRL + Qt::Key_Q);
    ui->menubar->addMenu(menu);
    mainLayout = new QGridLayout(this);
    mainLayout->addWidget(treeWidget, 0, 0, 4, 1);
    mainLayout->addWidget(textEdit, 4, 0, 3, 1);
    mainLayout->addWidget(putton, 7, 0, 1, 1);
    mainLayout->addWidget(putton_download, 8, 0, 1, 1);
    mainWidget = new QWidget(this);
    mainWidget->setLayout(mainLayout);
    setCentralWidget(mainWidget);
    setWindowTitle("XML Reader");

    dock = new QDockWidget("Query result", this, Qt::Popup);
    dock->resize(600,400);
    textEditForDock = new QTextEdit(this);
    textEditForDock->setReadOnly(true);
    textEditForDock->hide();
    dock->setWidget(textEditForDock);
    dock->setFloating(true);
    dock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetClosable);
    dock->setAllowedAreas(Qt::NoDockWidgetArea);
    dock->hide();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::CloseWindow()
{
    QApplication::quit();
}

void MainWindow::ClearTree()
{
    if(currnet_file)
    {
        delete currnet_file;
        currnet_file = nullptr;
        currnet_pos = nullptr;
    }
    treeWidget->clear();
}

void MainWindow::OpenFile()
{
    QString filepath = QFileDialog::getOpenFileName(this, "Choose file", "File", "*.xml");
    if(!filepath.isEmpty())
    {
        QFile file(filepath);
        QXmlSimpleReader reader;
        QString filename = filepath.section("/",-1,-1);
        AddressBookParser handler(treeWidget, filename);
        QXmlInputSource source(&file);
        reader.setContentHandler(&handler);
        if (!reader.parse(source))
            (new QErrorMessage(this))->showMessage("Cannot parse file!");
        if(!currnet_file)
        {
            currnet_file = new QFile(filepath);
            currnet_pos = treeWidget->itemAt(0, 0);
            currnet_pos->setFont(0, QFont("Times", 8, QFont::Bold));
        }
    }
}

void MainWindow::ExecuteQuery()
{  
    if(currnet_file->open(QIODevice::ReadOnly))
    {
        QString query = textEdit->toPlainText();
        QXmlQuery xml_query(QXmlQuery::XQuery10);
        if(!xml_query.setFocus(currnet_file))
             (new QErrorMessage(this))->showMessage("Can not set focus!");
        xml_query.setQuery(query);

        if (!xml_query.isValid())
        {
            currnet_file->close();
            (new QErrorMessage(this))->showMessage("The query is not valid!");
        }
        else
        {
            QString answer;
            if (!xml_query.evaluateTo(&answer))
            {
                qDebug() << answer;
                currnet_file->close();
                (new QErrorMessage(this))->showMessage("Can not evaluate the query!");
            }
            else
            {
                currnet_file->close();
                textEditForDock->setText(answer);
                dock->show();
            }
        }
    }

}

void MainWindow::OpenQuery()
{
    QString filepath = QFileDialog::getOpenFileName(this, "Choose query-file", "File", "*.xq");
    if (!filepath.isEmpty())
    {
        QFile file(filepath);
        if (file.open(QIODevice::ReadOnly))
        {
            QString query = file.readAll();
            file.close();
            textEdit->setText(query);
            ExecuteQuery();
        }
        else
        {
            (new QErrorMessage(this))->showMessage("Cannot open file with query!");
        }
    }
}

void MainWindow::slotCustomMenuRequested(const QPoint& pos)
{
    if(treeWidget->currentItem() && !treeWidget->currentItem()->parent())
        menuCustom->exec(QCursor::pos());
}

void MainWindow::ActActive()
{
    QString _currentFileName =  currnet_file->fileName();
    _currentFileName.resize(currnet_file->fileName().lastIndexOf('/') + 1);
    _currentFileName += treeWidget->currentItem()->data(0,0).toString();
    currnet_file->setFileName(_currentFileName);
    currnet_pos->setFont(0, QFont("Times", 8));
    treeWidget->currentItem()->setFont(0, QFont("Times", 8, QFont::Bold));
    currnet_pos = treeWidget->currentItem();
}

void MainWindow::CloseFile()
{
    QString newFileName =  currnet_file->fileName();
    newFileName.resize(currnet_file->fileName().lastIndexOf('/') + 1);

    QModelIndex _point;
    if(treeWidget->currentIndex().row())
        _point = treeWidget->model()->index(treeWidget->currentIndex().row() - 1,
                                                          treeWidget->currentIndex().column());

    else
        _point = treeWidget->model()->index(treeWidget->currentIndex().row() + 1,
                                                          treeWidget->currentIndex().column());

    QString currnetActiveFile(treeWidget->currentItem()->text(0));
    delete treeWidget->currentItem();

    if(!_point.isValid())
    {
        delete currnet_file;
        currnet_file = nullptr;
    }
    else
    {
        treeWidget->setCurrentIndex(_point);
        newFileName += treeWidget->currentItem()->data(0,0).toString();
        qDebug() << newFileName;
        if(currnet_file->fileName().contains(currnetActiveFile))
        {
            currnet_file->setFileName(newFileName);
            treeWidget->currentItem()->setFont(0, QFont("Times", 8, QFont::Bold));
        }
        currnet_pos = treeWidget->currentItem();
    }
}
